% This Matlab program generates figure 5 from the paper. It uses
% the data sets annual_bootstrap.csv containing asset returns and
% annual_rate1.csv on aggregate consumption. See the data description in
% the paper for details. 


clear
tic

rng(1234);

%loading data
load annual_bootstrap.csv;

% moment parameters

V1=1.028476;
V2=1.079768;
SIGMA1=0.0010827196;
SIGMA2=0.0015408712;
SIGMA3=0.0236718955;

r_m=[V1; V2];

r_v=[SIGMA1, SIGMA2; SIGMA2, SIGMA3];    


annual_means = 1 + mean(annual_bootstrap)';
annual_variance = cov(annual_bootstrap);

r_m = annual_means;
r_v = annual_variance;

%Monte Carlo Iterations
N=100;    

%Grid Precision Parameters
MX=1000;     %Play around with this you will see the dislocation at the bottom disappears as MX grows
MY=700;      %I have also noticed that when MX is too large for example 5000, the bounds gets too close to the real curve
plotting_grid=100; %To create lower quality plots when precision is too much


%Generating the Grid
grid_mean=linspace(0.75,1.25,MX)'; 
grid_deviation=linspace(0,7,MY)';     

%Auxiliary Calculations

%r_v_i: Inverse of Sigma
%A B C HJ Bound Coefficients

r_v_i=inv(r_v);
A=r_m'*r_v_i*r_m;
B=r_m'*r_v_i*ones(2,1);
C=ones(1,2)*r_v_i*ones(2,1);

%Original Empirical HJ Curve:
chart_deviation=sqrt(A*(grid_mean).^2-2*B*(grid_mean)+C);





%Bootstrapping
[bootstat, bootsam]=bootstrp(N, @mean, annual_bootstrap);
        

% %block bootstrap
% blength = 5;
% yrs = length(annual_bootstrap);
% nblocks = ceil(yrs/blength);
% blocksam = round(yrs*rand(nblocks,N));
% block_diff = ceil(blength/2)-(1:blength)';
% bootsam = reshape(repmat(blocksam,1,blength)',N,nblocks*blength)' + repmat(block_diff,nblocks,N);
% bootsam = max(bootsam,ones(nblocks*blength,N));
% bootsam = min(bootsam,yrs*ones(nblocks*blength,N));


        treas=annual_bootstrap(:,1);
        treas=treas(bootsam);
        stock=annual_bootstrap(:,2);
        stock=stock(bootsam);
        
%Obtaining the Sampling Distribution of Mean Estimate   
mean_treas=(mean(treas))'+1;
mean_stock=(mean(stock))'+1;


random_curves=zeros(MX,N);

%Variables to Save Distributions of Estimates of Return Moments
v1=zeros(1,N);
v2=zeros(1,N);
sigma1=zeros(1,N);
sigma2=zeros(1,N);
sigma3=zeros(1,N);

A_r=zeros(1,N);
B_r=zeros(1,N);
C_r=zeros(1,N);



for i=1:N
     
    r_m_random=[mean_treas(i), mean_stock(i)]';
    
     %calculating Estimated Variance Covariance Matrix for Each Sample
    r_v_random=cov(treas(:,i),stock(:,i),1);
    
    
    %These values are saved to estimate omega which is necessary in 
    %weight calculations
    v1(i)=r_m_random(1);
    v2(i)=r_m_random(2);
    sigma1(i)=r_v_random(1,1);
    sigma2(i)=r_v_random(1,2);
    sigma3(i)=r_v_random(2,2);
    %Now random_curves will save an HJ curve for each bootstrapped sample
    
    r_v_i_r=inv(r_v_random);
    A_r=r_m_random'*r_v_i_r*r_m_random;
    B_r=r_m_random'*r_v_i_r*ones(2,1);
    C_r=ones(1,2)*r_v_i_r*ones(2,1);
  
      
    random_curves(:,i)=sqrt(A_r*(grid_mean).^2-2*B_r*(grid_mean)+C_r);
    

end


%Calculating the gradient

[X,Y]=meshgrid(grid_mean,grid_deviation);
        F=sqrt(A*(X).^2-2*B*(X)+C)-Y;
        


[FX,FY]=gradient(F);



%Note that we only care about the gradient at the HJ Bound however the
%above formula gives us derivatives 
%for every point at grid_meanxgrid_deviation area.
%For any grid_mean value we search for the grid_deviation value to make
%sure that we are approximately at the HJ bound
%Hence m_derix and v_deriv saves derivative values along the HJ curve
[mino,deviation_ind]=min(abs(grid_deviation(:,ones(1,MX))-(chart_deviation(:,ones(1,MY)))'));
m_deriv=(diag(FX(deviation_ind,1:MX)))';
v_deriv=(diag(FY(deviation_ind,1:MX)))';
    


    

    
    
%Estimating Omega
gamma=[v1-1.028476; v2-1.079768; sigma1-0.00108271196; sigma2-0.0015408712; sigma3-0.0236718955];
omega=gamma*gamma'/N;    
omega_root=omega^(1/2);


M_partial_v1=zeros(MX,1);
M_partial_v2=zeros(MX,1);
M_partial_sigma1=zeros(MX,1);
M_partial_sigma2=zeros(MX,1);
M_partial_sigma3=zeros(MX,1);
weight=zeros(MX,1);

%calculating the weight

    
    A_partial_v1=(2*V1*SIGMA3-2*V2*SIGMA2)/(SIGMA1*SIGMA3-SIGMA2^2);
    A_partial_v2=(2*V2*SIGMA1-2*V1*SIGMA2)/(SIGMA1*SIGMA3-SIGMA2^2);
    A_partial_sigma1=-(((V2*SIGMA2-V1*SIGMA3)/(SIGMA1*SIGMA3-SIGMA2^2))^2);
    A_partial_sigma2=(-2*V1*V2*SIGMA1*SIGMA3+2*(V1^2)*SIGMA2*SIGMA3-2*V1*V2*(SIGMA2^2)+2*(V2^2)*SIGMA1*SIGMA2)/((SIGMA1*SIGMA3-SIGMA2^2)^2);
    A_partial_sigma3=-(((V1*SIGMA2-V2*SIGMA1)/(SIGMA1*SIGMA3-SIGMA2^2))^2);
    
    B_partial_v1=(SIGMA3-SIGMA2)/(SIGMA1*SIGMA3-SIGMA2^2);
    B_partial_v2=(SIGMA1-SIGMA2)/(SIGMA1*SIGMA3-SIGMA2^2);
    
    B_partial_sigma1=(-V2*(SIGMA2^2)-V1*(SIGMA3^2)+V2*SIGMA2*SIGMA3+V1*SIGMA2*SIGMA3)/(((SIGMA1*SIGMA3-SIGMA2^2))^2);
    
    B_partial_sigma2=(-V1*SIGMA1*SIGMA3-V2*SIGMA1*SIGMA3+2*V1*SIGMA2*SIGMA3-V2*(SIGMA2^2)-V1*(SIGMA2^2)+2*V2*SIGMA1*SIGMA2)/(((SIGMA1*SIGMA3-SIGMA2^2))^2);
    
    B_partial_sigma3=(-V1*(SIGMA2^2)+V2*SIGMA1*SIGMA2+V1*SIGMA1*SIGMA2-V2*(SIGMA1^2))/(((SIGMA1*SIGMA3-SIGMA2^2))^2);
    
    
    C_partial_v1=0;
    C_partial_v2=0;
    
    C_partial_sigma1=-(((SIGMA2-SIGMA3)/(SIGMA1*SIGMA3-SIGMA2^2))^2);
    
    C_partial_sigma2=(-2*SIGMA1*SIGMA3+2*SIGMA2*SIGMA3-2*(SIGMA2^2)+2*SIGMA1*SIGMA2)/(((SIGMA1*SIGMA3-SIGMA2^2))^2);
    
    C_partial_sigma3=-(((SIGMA2-SIGMA1)/(SIGMA1*SIGMA3-SIGMA2^2))^2);
    
    
    for j=1:MX
        
        M_partial_v1(j)=(A_partial_v1*grid_mean(j)^2-2*B_partial_v1*grid_mean(j)+C_partial_v1)...
                /(2*chart_deviation(j));
            
            M_partial_v2(j)=(A_partial_v2*grid_mean(j)^2-2*B_partial_v2*grid_mean(j)+C_partial_v2)...
                /(2*chart_deviation(j));
            
            M_partial_sigma1(j)=(A_partial_sigma1*grid_mean(j)^2-2*B_partial_sigma1*grid_mean(j)+C_partial_sigma1)...
                /(2*chart_deviation(j));
            
            M_partial_sigma2(j)=(A_partial_sigma2*grid_mean(j)^2-2*B_partial_sigma2*grid_mean(j)+C_partial_sigma2)...
                /(2*chart_deviation(j));
            
            M_partial_sigma3(j)=(A_partial_sigma3*grid_mean(j)^2-2*B_partial_sigma3*grid_mean(j)+C_partial_sigma3)...
                /(2*chart_deviation(j));
            
            weight(j)=(sqrt(sum(([M_partial_v1(j) M_partial_v2(j) M_partial_sigma1(j) M_partial_sigma2(j) M_partial_sigma3(j)]*omega_root).^2)))...
                /sqrt(((((A*grid_mean(j)-B)/chart_deviation(j))^2)+1));
              
    end 
    
    index_tracker=zeros(MX,N);
   
%Hausdorff Distance


total_distance=zeros(1,N);
    
for i=1:N
%For each Bootstrapped Curve We Create 2 (MX)x(MX) distance Matrices

%One is For X axis
X_difference=grid_mean(:,ones(1,MX))-grid_mean(:,ones(1,MX))';
Y=random_curves(:,i);

%The Other One Is For Y axis
Y_difference=(Y(:,ones(1,MX))-chart_deviation(:,ones(1,MX))');
X1_difference=X_difference;
X2_difference=X_difference;
Y1_difference=Y_difference;
Y2_difference=Y_difference;

    
%We Only Care When We are Below the True Curve So For Other Cases We Assign
%Zero Vertical Distance
    for j=1:MX
            if Y(j)<chart_deviation(j)
               
            X2_difference(:,j)=0;
            Y2_difference(:,j)=0;
        else
           
              
            X1_difference(j,:)=0;
            Y1_difference(j,:)=0;
            end
    end
   

%total_distance saves the Hausdorff Distance For Each Sample
total_distance(i)=max(min((sqrt(X1_difference.^2+Y1_difference.^2)) ,[],2));


    
total_distance(i)=max(min((sqrt(X_difference.^2+Y_difference.^2)) ,[],2 ));
[weighted_total_distance1(:,i), helper_index]=min((sqrt(X1_difference.^2+Y1_difference.^2)) ,[],2);

index_tracker(:,i)=helper_index; %helper_index is used to allow us to base weights
                                 %on the points at the original curve
end

weighted_total_distance2=weighted_total_distance1./(weight(index_tracker));
aux1=max(weighted_total_distance2);
aux2=sort(aux1);


weighted_h_distance_10=aux2(19*N/20);

weight_mean_10=grid_mean'+weighted_h_distance_10*(weight').*m_deriv./sqrt(m_deriv.^2+v_deriv.^2);
weight_stdev_10=chart_deviation'+weighted_h_distance_10*(weight').*v_deriv./sqrt(m_deriv.^2+v_deriv.^2);




mu0 = B/A;
max_mean_neg = max(weight_mean_10(weight_mean_10<mu0&weight_stdev_10>0));
min_mean_pos = min(weight_mean_10(weight_mean_10>mu0&weight_stdev_10>0));
max_std_neg = max(weight_stdev_10(weight_mean_10==max_mean_neg*ones(1,MX)));
max_std_pos = max(weight_stdev_10(weight_mean_10==min_mean_pos*ones(1,MX)));

weight_mean_10(weight_mean_10<max_mean_neg&weight_stdev_10<=max_std_neg) = max_mean_neg;
%weight_stdev_10(weight_mean_10<max_mean_neg&weight_stdev_10<=max_std_neg) = 0;
weight_mean_10(weight_mean_10>min_mean_pos&weight_stdev_10<=max_std_pos) = min_mean_pos;
%weight_stdev_10(weight_mean_10>min_mean_pos&weight_stdev_10<=max_std_pos) = 0;



weight_mean_10(weight_mean_10<=min_mean_pos+0.01&weight_mean_10>=max_mean_neg-0.01) = mu0;
weight_stdev_10(weight_mean_10==mu0) = 0.1;



% this is only done to fix issues with the appearance of the graphs
% since the projected points are very sparse around mu_0


ratio=MX/plotting_grid;

for k=1:plotting_grid
    
   GM(k)= grid_mean(k*ratio);
   RC(k,:)=random_curves(k*ratio,:);
   CD(k)=chart_deviation(k*ratio);
   WM(k)=weight_mean_10(k*ratio);
   WS(k)=weight_stdev_10(k*ratio);
    
    
    
end



% SDF implied by consumption data

%annual_rate = nondurables
%annual_rate1 = all. 

load annual_rate1.csv;

nc = length(annual_rate1);

annual_rate = ones(nc,1)./annual_rate1;

c_rate=annual_rate;
yrs = length(c_rate);

gamma_grd = 100;
gamma = (1:gamma_grd);
gamma(1) = 0;
M_mat = 0.95*repmat(c_rate,1,gamma_grd).^repmat(gamma,yrs,1);
SDF_mu = mean(M_mat);
SDF_sd = std(M_mat);

B = 600;
[bootstat, bootsam]=bootstrp(B, @mean, annual_rate);

% % block bootstrap
% blength = 5;
% nblocks = ceil(yrs/blength);
% blocksam = round(yrs*rand(nblocks,B));
% block_diff = ceil(blength/2)-(1:blength)';
% bootsam = reshape(repmat(blocksam,1,blength)',B,nblocks*blength)' + repmat(block_diff,nblocks,B);
% bootsam = max(bootsam,ones(nblocks*blength,B));
% bootsam = min(bootsam,yrs*ones(nblocks*blength,B));
% yrs = nblocks*blength;

crate_bs=annual_rate(:,1);
crate_bs=crate_bs(bootsam);

diff_random = zeros(gamma_grd,B);



gamma2_grd = 2000;
gamma2 = 2*(1:gamma2_grd);
gamma2(1) = 0;



%drawing hausdorff distance
for i=1:B
     
    M_mat_random = 0.95*repmat(crate_bs(:,i),1,gamma2_grd).^repmat(gamma2,yrs,1);
    SDF_mu_random = mean(M_mat_random);
    SDF_sd_random = std(M_mat_random);
    
    notzero = SDF_sd_random~=0;
    
    SDF_mu_interp = interp1(SDF_sd_random(notzero),SDF_mu_random(notzero),SDF_sd);
    diff_random(:,i) = SDF_mu_interp - SDF_mu;
    
    SDF_mu_interp_random(:,i) = SDF_mu_interp;
    
end

local_dev = std(diff_random,0,2)';

upper_dev = std(max(0,diff_random),0,2)';
lower_dev = std(min(0,diff_random),0,2)';

%local_dev = ones(1,gamma_grd);

weight_vert_dist = max(abs(diff_random)./repmat(local_dev',1,B));
weight_vert_dist = sort(weight_vert_dist);
crit_val = weight_vert_dist(1,0.95*B);

weight_upper_dist = max(max(0,diff_random)./repmat(upper_dev',1,B));
weight_upper_dist = sort(weight_upper_dist);
crit_upper = weight_upper_dist(1,0.975*B);

weight_lower_dist = max(max(0,-diff_random)./repmat(lower_dev',1,B));
weight_lower_dist = sort(weight_lower_dist);
crit_lower = weight_lower_dist(1,0.975*B);


SDF_mu_lb = SDF_mu - crit_val*local_dev;
SDF_mu_ub = SDF_mu + crit_val*local_dev;

SDF_mu_lb = SDF_mu - crit_lower*lower_dev;
SDF_mu_ub = SDF_mu + crit_upper*upper_dev;





figure
p1=plot(GM,RC,':'); hold on 
p3=plot(SDF_mu,SDF_sd,':r','LineWidth',2.5);
p6 = plot(SDF_mu_lb,SDF_sd,'--r','LineWidth',1.2);
p7 = plot(SDF_mu_ub,SDF_sd,'--r','LineWidth',1.2);
p2=plot(GM,CD,'-k','LineWidth',2.5);
p4 = plot(WM,WS,'--b','LineWidth',2.5); hold off
legend([p2' p4'],'Original Frontier','95% Confidence Region using Weighted W-Statistic','Location','NorthWest');
ylabel('standard deviation');   
M = [gamma',SDF_mu',SDF_sd'];
sel = [1 5 10 20 30 50 60 70 80 90 95];
M_sel=M(sel,:);
l = M_sel(:,1);
x = M_sel(:,2);
y = M_sel(:,3);
lbl_dwn = 0.02*6;

i = 1:length(sel);
text(x(i),y(i)+lbl_dwn,num2str(l(i)));

axis([0.4 1.25 0 6]);

















