% This Matlab program generates figure 6 from the paper. It uses
% the data sets annual_bootstrap.csv containing asset returns and
% annual_rate1.csv on aggregate consumption. See the data description in
% the paper for details. 



clear;
rng(1234);

tic


%loading data
load annual_bootstrap.csv;

% moment parameters
annual_means = 1 + mean(annual_bootstrap)';
annual_variance = cov(annual_bootstrap);

%Monte Carlo Iterations
N=100;    %Originally I have done 10000 to calculate the distribution of Hausdorff distance 
          %however that takes time

%Grid Parameters
MX=500;   %1000 for monthly
MY=700;   %6000 for monthly
MD=600;
sharpe_ratio=1;



%Choice between monthly and annual
%Also need to change the bootstrap part to switch between years and months
r_m=annual_means;
r_v=annual_variance;
observations=length(annual_bootstrap);  %48 for annual  576 for monthly
grid_mean=linspace(0.75,1.25,MX)'; % annual: 0.92-1.08  monthly: 0.997-1.002

%grid_deviation will only be useful when calculating the gradient
grid_deviation=linspace(0,7,MY)';     
grid_derivative=linspace(0,0.16,MX)';

%Auxiliary Calculations
r_v_i=inv(r_v);
A=r_m'*r_v_i*r_m;
B=r_m'*r_v_i*ones(2,1);
C=ones(1,2)*r_v_i*ones(2,1);

%Initial Curve
chart_variance=A*(grid_mean).^2-2*B*(grid_mean)+C;
  
chart_deviation=sqrt(chart_variance);




%Bootstrapping
[bootstat, bootsam]=bootstrp(N, @mean, annual_bootstrap);


% % block bootstrap
% blength = 5;
% yrs = length(annual_bootstrap);
% nblocks = ceil(yrs/blength);
% blocksam = round(yrs*rand(nblocks,N));
% block_diff = ceil(blength/2)-(1:blength)';
% bootsam = reshape(repmat(blocksam,1,blength)',N,nblocks*blength)' + repmat(block_diff,nblocks,N);
% bootsam = max(bootsam,ones(nblocks*blength,N));
% bootsam = min(bootsam,yrs*ones(nblocks*blength,N));



        treas=annual_bootstrap(:,1);
        treas=treas(bootsam);
        stock=annual_bootstrap(:,2);
        stock=stock(bootsam);
        
  
mean_treas=(mean(treas))'+1;
mean_stock=(mean(stock))'+1;

difference=ones(N,MX);


%drawing hausdorff distance
for i=1:N
     
    r_m_random=[mean_treas(i), mean_stock(i)]';
    r_v_random=cov(treas(:,i),stock(:,i),1);
    
    r_v_i_r=inv(r_v_random);
    A_r=r_m_random'*r_v_i_r*r_m_random;
    B_r=r_m_random'*r_v_i_r*ones(2,1);
    C_r=ones(1,2)*r_v_i_r*ones(2,1);
    r_m_random=[mean_treas(i), mean_stock(i)]';
    r_v_random=cov(treas(:,i),stock(:,i),1);
      
    chart_deviation_random=sqrt(A_r*(grid_mean).^2-2*B_r*(grid_mean)+C_r);
    
    difference(i,:)=(chart_deviation-chart_deviation_random);
    

end

L=sqrt(1+((((grid_mean).*A-B).^2)./((chart_deviation).^2)));



difference_abs=abs(difference);
hausdorf=difference_abs*inv(diag(L));
hausdorf_uniform=max(hausdorf,[],2);

aux1=sort(difference_abs);
aux2=sort(hausdorf);
aux3=sort(hausdorf_uniform);

value_difference_1=aux1(99*N/100,:);
value_difference_5=aux1(19*N/20,:);
value_difference_10=aux1(9*N/10,:);

h_distance_1=aux2(99*N/100,:);
h_distance_5=aux2(19*N/20,:);
h_distance_10=aux2(9*N/10,:);

hu_distance_1=aux3(99*N/100);
hu_distance_5=aux3(19*N/20);
hu_distance_10=aux3(9*N/10);





%Calculating the gradient

[X,Y]=meshgrid(grid_mean,grid_deviation);
        F=sqrt(A*(X).^2-2*B*(X)+C)-Y;
        


[FX,FY]=gradient(F);



%Matching the standard deviation grid to the function
[mino,deviation_ind]=min(abs(grid_deviation(:,ones(1,MX))-(chart_deviation(:,ones(1,MY)))'));
m_deriv=(diag(FX(deviation_ind,1:MX)))';
v_deriv=(diag(FY(deviation_ind,1:MX)))';
    

%monthly hausdorff distances: 10%: 0.0815 5%: 0.1064 1%: 0.1546
%annual hausdorff distances: 10%: 1.0567 5%: 1.4021 1%:1.9965 

%Drawing the initial curve and the confidence interval

    new_m_1=grid_mean'+h_distance_1.*m_deriv./sqrt(m_deriv.^2+v_deriv.^2);
    new_s_1=chart_deviation'+h_distance_1.*v_deriv./sqrt(m_deriv.^2+v_deriv.^2);
    new_m_5=grid_mean'+h_distance_5.*m_deriv./sqrt(m_deriv.^2+v_deriv.^2);
    new_s_5=chart_deviation'+h_distance_5.*v_deriv./sqrt(m_deriv.^2+v_deriv.^2);
    new_m_10=grid_mean'+h_distance_10.*m_deriv./sqrt(m_deriv.^2+v_deriv.^2);
    new_s_10=chart_deviation'+h_distance_10.*v_deriv./sqrt(m_deriv.^2+v_deriv.^2);
    
    new_mu_1=grid_mean'+hu_distance_1*m_deriv./sqrt(m_deriv.^2+v_deriv.^2);
    new_su_1=chart_deviation'+hu_distance_1*v_deriv./sqrt(m_deriv.^2+v_deriv.^2);
    new_mu_5=grid_mean'+hu_distance_5*m_deriv./sqrt(m_deriv.^2+v_deriv.^2);
    new_su_5=chart_deviation'+hu_distance_5*v_deriv./sqrt(m_deriv.^2+v_deriv.^2);
    new_mu_10=grid_mean'+hu_distance_10*m_deriv./sqrt(m_deriv.^2+v_deriv.^2);
    new_su_10=chart_deviation'+hu_distance_10*v_deriv./sqrt(m_deriv.^2+v_deriv.^2);

%Some Diagnostics

random_curves=chart_deviation(:,ones(1,N))-difference';

total_distance=ones(1,N);
total_distance_t=ones(1,N);
total_distance1=ones(1,N);
total_distance2=ones(1,N);
total_distance3=ones(1,N);

for i=1:N
X_difference=grid_mean(:,ones(1,MX))-grid_mean(:,ones(1,MX))';
Y=random_curves(:,i);
Y_difference=(Y(:,ones(1,MX))-chart_deviation(:,ones(1,MX))')/sharpe_ratio;
X1_difference=X_difference;
X2_difference=X_difference;
Y1_difference=Y_difference;
Y2_difference=Y_difference;

    for j=1:MX
            if Y(j)<chart_deviation(j)
               
            X2_difference(:,j)=0;
            Y2_difference(:,j)=0;
        else
           
              
            X1_difference(j,:)=0;
            Y1_difference(j,:)=0;
            end
    end
   

total_distance(i)=max(min((sqrt(X_difference.^2+Y_difference.^2)) ,[],2 ));
total_distance_t(i)=max(min((sqrt(X_difference.^2+Y_difference.^2)) ,[],1 ));
total_distance1(i)=max(min((sqrt(X1_difference.^2+Y1_difference.^2)) ,[],2));
total_distance2(i)=max(min((sqrt(X2_difference.^2+Y2_difference.^2)) ,[],1));
total_distance3(i)=max(total_distance1(i),total_distance2(i));
end



aux=sort(total_distance1);
emp_h_distance_10=aux(19*N/20);
new_em_10=grid_mean'+emp_h_distance_10*m_deriv./sqrt(m_deriv.^2+v_deriv.^2);
new_es_10=chart_deviation'+sharpe_ratio*emp_h_distance_10*v_deriv./sqrt(m_deriv.^2+v_deriv.^2);



% SDF implied by consumption data

%annual_rate = nondurables
%annual_rate1 = all. 

load annual_rate1.csv;

nc = length(annual_rate1);

annual_rate = ones(nc,1)./annual_rate1;

c_rate=annual_rate;
yrs = length(c_rate);

gamma_grd = 100;
gamma = (1:gamma_grd);
gamma(1) = 0;
M_mat = 0.95*repmat(c_rate,1,gamma_grd).^repmat(gamma,yrs,1);
SDF_mu = mean(M_mat);
SDF_sd = std(M_mat);

B = 600;
[bootstat, bootsam]=bootstrp(B, @mean, annual_rate);

% %block bootstrap
% blength = 5;
% nblocks = ceil(yrs/blength);
% blocksam = round(yrs*rand(nblocks,B));
% block_diff = ceil(blength/2)-(1:blength)';
% bootsam = reshape(repmat(blocksam,1,blength)',B,nblocks*blength)' + repmat(block_diff,nblocks,B);
% bootsam = max(bootsam,ones(nblocks*blength,B));
% bootsam = min(bootsam,yrs*ones(nblocks*blength,B));
% yrs = nblocks*blength;


crate_bs=annual_rate(:,1);
crate_bs=crate_bs(bootsam);

diff_random = zeros(gamma_grd,B);



gamma2_grd = 2000;
gamma2 = 2*(1:gamma2_grd);
gamma2(1) = 0;



%drawing hausdorff distance
for i=1:B
     
    M_mat_random = 0.95*repmat(crate_bs(:,i),1,gamma2_grd).^repmat(gamma2,yrs,1);
    SDF_mu_random = mean(M_mat_random);
    SDF_sd_random = std(M_mat_random);
    
    notzero = SDF_sd_random~=0;
    
    SDF_mu_interp = interp1(SDF_sd_random(notzero),SDF_mu_random(notzero),SDF_sd);
    diff_random(:,i) = SDF_mu_interp - SDF_mu;
    
    
end

local_dev = std(diff_random,0,2)';

upper_dev = std(max(0,diff_random),0,2)';
lower_dev = std(min(0,diff_random),0,2)';

%local_dev = ones(1,gamma_grd);

weight_vert_dist = max(abs(diff_random)./repmat(local_dev',1,B));
weight_vert_dist = sort(weight_vert_dist);
crit_val = weight_vert_dist(1,0.95*B);

weight_upper_dist = max(max(0,diff_random)./repmat(upper_dev',1,B));
weight_upper_dist = sort(weight_upper_dist);
crit_upper = weight_upper_dist(1,0.975*B);

weight_lower_dist = max(max(0,-diff_random)./repmat(lower_dev',1,B));
weight_lower_dist = sort(weight_lower_dist);
crit_lower = weight_lower_dist(1,0.975*B);


SDF_mu_lb = SDF_mu - crit_val*local_dev;
SDF_mu_ub = SDF_mu + crit_val*local_dev;

SDF_mu_lb = SDF_mu - crit_lower*lower_dev;
SDF_mu_ub = SDF_mu + crit_upper*upper_dev;




figure
p1=plot(grid_mean,random_curves,':'); hold on 
p3=plot(SDF_mu,SDF_sd,':r','LineWidth',2.5);
p4 = plot(SDF_mu_lb,SDF_sd,'--r','LineWidth',1.2);
p5 = plot(SDF_mu_ub,SDF_sd,'--r','LineWidth',1.2);
p2=plot(grid_mean,chart_deviation,'-k.',new_em_10,new_es_10, '--b','LineWidth',2.5); hold off
legend([p2' p1'],'Original Frontier','95% Confidence Set using Unweighted W-Statistic','Location','NorthWest');
xlabel('mean');
ylabel('standard deviation');   
M = [gamma',SDF_mu',SDF_sd'];
sel = [1 5 10 20 30 50 60 70 80 90 95];
M_sel=M(sel,:);
l = M_sel(:,1);
x = M_sel(:,2);
y = M_sel(:,3);
lbl_dwn = 0.02*6;
i = 1:length(sel);
text(x(i),y(i)+lbl_dwn,num2str(l(i)));
axis([0.4 1.25 0 6]);


toc    
    






















