% This Matlab program generates figures 3,4, and 7 from the paper. It uses
% the data sets annual_bootstrap.csv containing asset returns and
% annual_rate1.csv on aggregate consumption. See the data description in
% the paper for details. 



clear;
alpha = 0.05;
rng(1234);





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Weighted and Unweighted LR-Statistic %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%loading data
load annual_bootstrap.csv;


% moment parameters
annual_means = 1 + mean(annual_bootstrap)';
annual_variance = cov(annual_bootstrap);

%Monte Carlo Iterations
N=100;    %Originally I have done 10000 to calculate the distribution of Hausdorff distance 
          %however that takes time

%Grid Parameters
MX=500;   %1000 for monthly
MY=700;   %6000 for monthly
MD=600;
z_gd = 100;
plotting_grid = 500;


%Choice between monthly and annual
%Also need to change the bootstrap part to switch between years and months
r_m=annual_means;
r_v=annual_variance;
observations=length(annual_bootstrap);  %48 for annual  576 for monthly
grid_mean=linspace(0.4,1.25,MX)'; % annual: 0.92-1.08  monthly: 0.997-1.002
grid_deviation=linspace(0,7,MY)';     
grid_derivative=linspace(0,0.16,MX)';

%Auxiliary Calculations
r_v_i=inv(r_v);
A=r_m'*r_v_i*r_m;
B=r_m'*r_v_i*ones(2,1);
C=ones(1,2)*r_v_i*ones(2,1);

A_bs = zeros(N,1);
B_bs = zeros(N,1);
C_bs = zeros(N,1);

%Initial Curve
chart_variance=A*(grid_mean).^2-2*B*(grid_mean)+C;
  
chart_deviation=sqrt(chart_variance);




%Bootstrapping
[bootstat, bootsam]=bootstrp(N, @mean, annual_bootstrap);



% block bootstrap
blength = 3;
yrs = length(annual_bootstrap);
nblocks = ceil(yrs/blength);
blocksam = round(yrs*rand(nblocks,N));
block_diff = ceil(blength/2)-(1:blength)';
bootsam = reshape(repmat(blocksam,1,blength)',N,nblocks*blength)' + repmat(block_diff,nblocks,N);
bootsam = max(bootsam,ones(nblocks*blength,N));
bootsam = min(bootsam,yrs*ones(nblocks*blength,N));



        
        treas=annual_bootstrap(:,1);
        treas=treas(bootsam);
        stock=annual_bootstrap(:,2);
        stock=stock(bootsam);
        
  
mean_treas=(mean(treas))'+1;
mean_stock=(mean(stock))'+1;

L=ones(N,MX);
difference=ones(N,MX);
first_derivx=ones(N,MX);
second_derivx=ones(N,MX);
chart_deviation_random=ones(N,MX);

%drawing hausdorff distance
for i=1:N
     
    r_m_random=[mean_treas(i), mean_stock(i)]';
    r_v_random=cov(treas(:,i),stock(:,i),1);
    
    r_v_i_r=inv(r_v_random);
    A_r=r_m_random'*r_v_i_r*r_m_random;
    B_r=r_m_random'*r_v_i_r*ones(2,1);
    C_r=ones(1,2)*r_v_i_r*ones(2,1);
    r_m_random=[mean_treas(i), mean_stock(i)]';
    r_v_random=cov(treas(:,i),stock(:,i),1);
      
    chart_deviation_random(i,:)=sqrt(A_r*(grid_mean).^2-2*B_r*(grid_mean)+C_r);
    
    %Calculating the derivative in Molchanov
    %Derivation for the formula for L(j) can be found in the 
    %"calculations for the code file"
  
    difference(i,:)=chart_deviation_random(i,:)-chart_deviation';
    A_bs(i,1) = A_r;
    B_bs(i,1) = B_r;
    C_bs(i,1) = C_r;
    
    
end


precision=std(difference);
difference_abs=abs(difference);

max_difference=max(difference_abs, [],2);
min_difference=min(difference, [],2);

hausdorff_distance=difference*(inv(diag(precision)));
uniform_hausdorf=min(hausdorff_distance,[],2);


   




aux=sort(difference);
aux2=sort(difference_abs);
aux3=sort(max_difference);
aux4=sort(uniform_hausdorf);
aux5=sort(min_difference);


distance_abs1=aux2(99*N/100,:)';
distance_abs5=aux2(19*N/20,:)';
distance_abs10=aux2(9*N/10,:)';

udistance_1=aux3(99*N/100,:)';
udistance_5=aux3(19*N/20,:)';
udistance_10=aux3(9*N/10,:)';

udistance_down5=aux5(N/20);
udistance_down10=aux5(N/10);

hdistance_1=aux4(N/100);
hdistance_5=aux4(N/20);
hdistance_10=aux4(N/10);

hdistance_alpha=aux4(alpha*N);



%Drawing Horizontal Distance Vertical Distance Frontiers

vertical_down5=chart_deviation+udistance_down5;

precision_down_alpha=chart_deviation+hdistance_alpha*precision';






%%%%%%%%%%%%%%%%%%%%%%%%
% Projection approach: %         
%%%%%%%%%%%%%%%%%%%%%%%%


% generate grid on the variance ellipsoid for (A,B,C)

Bs_abc = [A_bs-A,B_bs-B,C_bs-C];
Z_var = cov([A_bs,B_bs,C_bs]);

chisq_bs = diag(Bs_abc*inv(Z_var)*Bs_abc');
chisq_bs = sort(chisq_bs);
chisq_pct = chisq_bs((1-alpha)*N);


% generate three-dimensional sphere S
theta = pi*(1:z_gd)/z_gd;
phi = 2*acos(2*(1:z_gd)/z_gd-1);


Sgrid1 = reshape(cos(phi)'*sin(theta),1,z_gd^2);
Sgrid2 = reshape(sin(phi)'*sin(theta),1,z_gd^2);
Sgrid3 = reshape(repmat(cos(theta),z_gd,1),1,z_gd^2);

S_grid = [Sgrid1;Sgrid2;Sgrid3];

% confidence ellipsoid for (A,B,C) using chi-square and bootstrap critical
% values

Z_grid = [A;B;C]*ones(1,z_gd^2) + sqrt(chi2inv(1-alpha,3))*sqrtm(Z_var)*S_grid;   


% Projection approach: generate bounds on the standard deviation

grid_std = sqrt(max(((grid_mean).^2)*Z_grid(1,:)-2*grid_mean*Z_grid(2,:)+repmat(Z_grid(3,:),MX,1),0));
proj_stdev_10 = min(grid_std')';

sel_binding = max(grid_std==repmat(proj_stdev_10,1,z_gd^2));
gamma_curve = Z_grid(repmat(sel_binding,3,1));

% Projection approach: plot figures

ratio=MX/plotting_grid;

for k=1:plotting_grid

   PM(k)= grid_mean(k*ratio);
   PS(k)= proj_stdev_10(k*ratio);
    
    
    
end




% SDF implied by consumption data 

load annual_rate1.csv;

nc = length(annual_rate1);

annual_rate = ones(nc,1)./annual_rate1;

c_rate=annual_rate;
yrs = length(c_rate);

gamma_grd = 100;
gamma = (1:gamma_grd);
gamma(1) = 0;
M_mat = 0.95*repmat(c_rate,1,gamma_grd).^repmat(gamma,yrs,1);
SDF_mu = mean(M_mat);
SDF_sd = std(M_mat);

B = 600;
[bootstat, bootsam]=bootstrp(B, @mean, annual_rate);


% blength = 5;
nblocks = ceil(yrs/blength);
blocksam = round(yrs*rand(nblocks,B));
block_diff = ceil(blength/2)-(1:blength)';
bootsam = reshape(repmat(blocksam,1,blength)',B,nblocks*blength)' + repmat(block_diff,nblocks,B);
bootsam = max(bootsam,ones(nblocks*blength,B));
bootsam = min(bootsam,yrs*ones(nblocks*blength,B));
yrs = nblocks*blength;


crate_bs=annual_rate(:,1);
crate_bs=crate_bs(bootsam);

diff_random = zeros(gamma_grd,B);



gamma2_grd = 2000;
gamma2 = 2*(1:gamma2_grd);
gamma2(1) = 0;



%drawing hausdorff distance
for i=1:B
     
    M_mat_random = 0.95*repmat(crate_bs(:,i),1,gamma2_grd).^repmat(gamma2,yrs,1);
    SDF_mu_random = mean(M_mat_random);
    SDF_sd_random = std(M_mat_random);
    
    notzero = SDF_sd_random~=0;
    
    SDF_mu_interp = interp1(SDF_sd_random(notzero),SDF_mu_random(notzero),SDF_sd);
    diff_random(:,i) = SDF_mu_interp - SDF_mu;
    
    
end

local_dev = std(diff_random,0,2)';

upper_dev = std(max(0,diff_random),0,2)';
lower_dev = std(min(0,diff_random),0,2)';

%local_dev = ones(1,gamma_grd);

weight_vert_dist = max(abs(diff_random)./repmat(local_dev',1,B));
weight_vert_dist = sort(weight_vert_dist);
crit_val = weight_vert_dist(1,0.95*B);

weight_upper_dist = max(max(0,diff_random)./repmat(upper_dev',1,B));
weight_upper_dist = sort(weight_upper_dist);
crit_upper = weight_upper_dist(1,0.975*B);

weight_lower_dist = max(max(0,-diff_random)./repmat(lower_dev',1,B));
weight_lower_dist = sort(weight_lower_dist);
crit_lower = weight_lower_dist(1,0.975*B);


SDF_mu_lb = SDF_mu - crit_val*local_dev;
SDF_mu_ub = SDF_mu + crit_val*local_dev;

SDF_mu_lb = SDF_mu - crit_lower*lower_dev;
SDF_mu_ub = SDF_mu + crit_upper*upper_dev;





figure
random_curves=chart_deviation(:,ones(1,N))-difference';
p1=plot(grid_mean,chart_deviation_random,':'); hold on 
p3=plot(SDF_mu,SDF_sd,':r','LineWidth',2.5);
p4 = plot(SDF_mu_lb,SDF_sd,'--r','LineWidth',1.2);
p5 = plot(SDF_mu_ub,SDF_sd,'--r','LineWidth',1.2);
p2=plot(grid_mean,chart_deviation,'-k',grid_mean, precision_down_alpha,'--b','LineWidth',3); hold off
legend([p2' p3' p5'],'Original Frontier','95% Confidence Region based on Weighted LR Statistic','SDF implied by Aggregate Consumption','95% Confidence Bands for Consumption Based SDF','Location','NorthWest');
xlabel('mean');
ylabel('standard deviation');   
M = [gamma',SDF_mu',SDF_sd'];
sel = [1 5 10 20 30 50 60 70 80 90 95];
M_sel=M(sel,:);
l = M_sel(:,1);
x = M_sel(:,2);
y = M_sel(:,3);
lbl_dwn = 0.02*6;
i = 1:length(sel);
text(x(i),y(i)+lbl_dwn,num2str(l(i)));
axis([0.4 1.25 0 6]);


figure
p1=plot(grid_mean,chart_deviation_random,':'); hold on 
p3=plot(SDF_mu,SDF_sd,':r','LineWidth',2.5);
p4 = plot(SDF_mu_lb,SDF_sd,'--r','LineWidth',1.2);
p5 = plot(SDF_mu_ub,SDF_sd,'--r','LineWidth',1.2);
p2=plot(grid_mean,chart_deviation,'-k',grid_mean, vertical_down5, '--b','LineWidth',3); hold off
legend([p2' p1'],'Original Frontier','95% Confidence Region based on Unweighted LR Statistic','Location','NorthWest');
xlabel('mean');
ylabel('standard deviation');
M = [gamma',SDF_mu',SDF_sd'];
sel = [1 5 10 20 30 50 60 70 80 90 95];
M_sel=M(sel,:);
l = M_sel(:,1);
x = M_sel(:,2);
y = M_sel(:,3);
lbl_dwn = 0.02*6;
i = 1:length(sel);
text(x(i),y(i)+lbl_dwn,num2str(l(i)));
axis([0.4 1.25 0 6]);


figure
p1=plot(PM,PS,'b--','LineWidth',1.5); hold on 
p5=plot(grid_mean,chart_deviation_random,':');
p3=plot(SDF_mu,SDF_sd,':r','LineWidth',2.5);
p6 = plot(SDF_mu_lb,SDF_sd,'--r','LineWidth',1.2);
p7 = plot(SDF_mu_ub,SDF_sd,'--r','LineWidth',1.2);
p2=plot(grid_mean,chart_deviation,'-k',grid_mean,precision_down_alpha,'-b','LineWidth',3); hold off
legend([p2' p1'],'Original Frontier','95% Confidence Set based on Weighted LR Statistic','95% Confidence Set based on Projection','Location','NorthWest');
xlabel('mean');
ylabel('standard deviation');   
M = [gamma',SDF_mu',SDF_sd'];
sel = [1 5 10 20 30 50 60 70 80 90 95];
M_sel=M(sel,:);
l = M_sel(:,1);
x = M_sel(:,2);
y = M_sel(:,3);
lbl_dwn = 0.02*6;
i = 1:length(sel);
text(x(i),y(i)+lbl_dwn,num2str(l(i)));
axis([0.4 1.25 0 6]);


% figure


% Z = [A;B;C]*ones(1,z_gd^2) + sqrt(chi2inv(1-alpha,3))*Z_var^(0.5)*S'; 
% p1 = surf(Z); hold on
% p2 = plot3(gamma_curve); hold off









   
    























